from termcolor import colored
import time
import sys
import pickle

class status_message(object):

    last_block = None


    def __init__(self, message):
        self.message = message
        self.parent = status_message.last_block
        status_message.last_block = self


    @property
    def depth(self):
        if self.parent is None:
            return 1
        else:
            return self.parent.depth + 1


    def __enter__(self):
        if self.parent is not None:
            print
        self.print_start()
        self.start_time = time.time()


    def print_start(self, lead="==>"):
        print (colored(lead * self.depth + " ", "blue", attrs=["bold"]) 
               + self.message 
               + "... "),
        self.line_len = 5 + self.depth * len(lead) + len(self.message)
        sys.stdout.flush()


    def format_time(self, secs):
        times = [('y', 365 * 24 * 60 * 60),
                 ('m', 30 * 24 * 60 * 60),
                 ('d', 24 * 60 * 60),
                 ('h', 60 * 60),
                 ('m', 60),
                 ('s', 1)]
        for n, t in times:
            if t <= secs:
                return "(%2i%s)" % (int(secs / t), n)
        else:
            return '(' + ("%.1fs)" % (secs / t))[1:]
        

    def __exit__(self, type, value, traceback):
        delta = time.time() - self.start_time
        if value is None:
            print (' ' * (75 - self.line_len - 5) 
                   + colored("done! ", "green", attrs=["bold"]) 
                   + self.format_time(delta))
        else:
            print (' ' * (75 - self.line_len - 7) 
                   + colored("failed! ", "red", attrs=["bold"]) 
                   + self.format_time(delta))

        if self.parent is not None:
            self.parent.print_start("...")
        status_message.last_block = self.parent


# def test():
#     import time
#     with status_message("something"):
#         time.sleep(5)
#     with status_message("something else"):
#         time.sleep(0.5)
#         try:
#             with status_message("another thing"):
#                 raise Exception("Oh shit!")
#         except:
#             pass
